#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#define M_PI	(3.141592653589793)
#define EPS  (1.0e-3)
double frand(void)
{
	double ret;
	int val = rand();
	while(val > 32767) {
		val = rand();
	}
	ret = ((double)val)/32767.0;
	return ret;
}

int compareDblGT(const void *pd1, const void *pd2)
{
	double d1, d2;
	d1 = *((double *)(pd1));
	d2 = *((double *)(pd2));
	if(d1 > d2) {
		return -1;
	} else if (d1 < d2) {
		return 1;
	} else {
		return 0;
	}
}

long AX[200], AY[200], BX[200], BY[200];
double Aang[200], Bang[200], Axd[200], Bxd[200], Ayd[200], Byd[200];
double Arad, Brad, X, Y, X1, Y1, X2, Y2;
int Apts, Bpts;

int MakeRandangles(double *pAngs, int nAng)
{
	int i;
	for(i = 0; i < nAng ; i++) {
		pAngs[i] = 2.0*frand()*M_PI;
	}
	qsort(pAngs, nAng, sizeof(double), compareDblGT);
	for(i = 0; i < nAng-1 ; i++) {
		if(pAngs[i+1] > (pAngs[i] - EPS)) {
			pAngs[i+1] = (pAngs[i] - EPS);
		}
	}
	if(pAngs[nAng-1] < (pAngs[0] - 2.0*M_PI + EPS)) {
		pAngs[nAng-1] = (pAngs[0] - 2.0*M_PI + EPS);
		i = nAng - 1;
		while(pAngs[i-1] < (pAngs[i]+EPS)) {
			pAngs[i-1] = (pAngs[i]+EPS);
			i--;
		}
	}

	return 0;
}

// ray x = xray + t*cos, y = yray + t*sin
// on circle (x -xcen)^2 + (y - ycen)^2 = rad^2 = (dx+t*cos)^2 + (dy + t*sin)^2
// dx^2 + 2*dx*cos*t + cos^2*t^2 + dy^2 + 2*dy*t*sin + sin^2*t^2 - rad^2 = 0
// (dx^2 + dy^2 - rad^2) + 2*(cos*dx +dy*sin)*t + (cos^2 + sin^2)*t*t = 0
int MakeDblPts(int npts, double *px, double *py, double *pAng,
			   double rad, double xcen, double ycen,
			   double xray, double yray)
{
	int i;
	double t, c, s, dx, dy, C, B;
	dx = xray - xcen;
	dy = yray - ycen;
	C = (dx*dx + dy*dy) - rad*rad;
	if(C > -100.0) {
		return -1;
	}
	for(i = 0; i < npts ; i++) {
		c = cos(pAng[i]);
		s = sin(pAng[i]);
		B = (c*dx + s*dy);
		t = (-B + sqrt(B*B - C));
		px[i] = xray + t*c;
		py[i] = yray + t*s;
	}
	return 0;
}

int MakeLongPts(int npts, long *px, long *py, double *pdx, double *pdy, double scale)
{
	int i;
	for(i = 0; i < npts; i++) {
		px[i] = (long)((scale*pdx[i]) + 0.5);
		py[i] = (long)((scale*pdy[i]) + 0.5);
	}
	return 0;
}

int CheckColinear(long *px, long *py, int npts) {
	int dx1, dx2, dy1, dy2, i;
	for(i = 0; i < npts; i++) {
		dx1 = px[(i+1)%npts] - px[i];
		dy1 = py[(i+1)%npts] - py[i];
		dx2 = px[(i+2)%npts] - px[(i+1)%npts];
		dy2 = py[(i+2)%npts] - py[(i+1)%npts];
		if((dx1 == dx2) && (dy1 == dy2)) {
			return -1;
		}
	}
	return 0;
}

// dy*(x-x0) -dx*(y-y0) = 0 => dy*x - dx*y - (dy*x0 - dx*y0) = 0 on line
// pos on one side neg on the other
double CheckOutside()
{
	double scale, A, B, C, dot, dy, dx, tmp;
	int i, j;
	scale = 0.0;
	for(i = 0; i < Apts; i++) {
		dy = Ayd[(i+1) % Apts] - Ayd[i];
		dx = Axd[(i+1) % Apts] - Axd[i];
		A = dy;
		B = -dx;
		C = -(A*Axd[i] +B*Ayd[i]);
		for(j = 0; j < Bpts; j++) {
			dot = A*((double)BX[j]) + B*((double)BY[j]) + C;
			tmp = (C - dot)/C;
			if(tmp > scale) {
				scale = tmp;
			}
		}
	}
	return scale;

}

void WriteProblem(FILE *fp)
{
	int i;
	fprintf(fp, "%d\n", Apts);
	for(i = 0 ; i < Apts ; i++) {
		fprintf(fp, "%d %d\n", AX[i], AY[i]);
	}
	fprintf(fp, "%d\n", Bpts);
	for(i = 0 ; i < Bpts ; i++) {
		fprintf(fp, "%d %d\n", BX[i], BY[i]);
	}
}

char namebuf[256];
int main(int argc, char **argv)
{
	int label, doshrink, i, colinear;
	double c, s, r, ang, scale;
	FILE *fp;
	if(argc < 4) {
		fprintf(stderr, "USAGE: %s Apts Bpts file_index [doshrink]\n");
		return -1;
	}
	Apts = atoi(argv[1]);
	if((Apts < 3) || (Apts > 200)) {
		fprintf(stderr, "Apts %d not in range 3 .. 200\n", Apts);
		return -2;
	}
	Bpts = atoi(argv[2]);
	if((Bpts < 3) || (Bpts > 200)) {
		fprintf(stderr, "Bpts %d not in range 3 .. 200\n", Bpts);
		return -3;
	}
	label = atoi(argv[3]);
	sprintf(&(namebuf[0]), "CutItOut-%04d.in", label);
	if((fp = fopen(namebuf, "w")) == NULL) {
		fprintf(stderr, "Cannot open file \"%s\" for wrtie\n", namebuf);
		return -4;
	}
	doshrink = 0;
	if(argc >= 5) {
		doshrink = atoi(argv[4]);
	}
	srand(time(NULL));
	colinear = 1;
	for(i = 0; i < 200; i++) {
		if(Apts == Bpts) {
			Arad = 1000000.0 + frand()*(1000000.0);
			Brad = 100000.0 + frand()*(1000000.0);
			if(Brad > (Arad - 100.0)) Brad = (Arad - 100.0);
			r = Arad - Brad;
			r = (0.25*r + 0.5*r*frand());
			ang = 2.0*M_PI*frand();
			c = cos(ang);
			s = sin(ang);
			X = r*c;
			Y = r*s;
			r = Brad - sqrt(X*X + Y*Y);
			r = (0.25*r + 0.5*r*frand());
			ang = 2.0*M_PI*frand();
			c = cos(ang);
			s = sin(ang);
			X1 = X + r*c;
			Y1 = Y + r*s;
			MakeRandangles(&(Aang[0]), Apts);
			MakeDblPts(Apts, &(Axd[0]), &(Ayd[0]), &(Aang[0]), Arad, 0.0, 0.0, X1, Y1);
			MakeDblPts(Apts, &(Bxd[0]), &(Byd[0]), &(Aang[0]), Brad, X, Y, X1, Y1);
			MakeLongPts(Apts, &(BX[0]), &(BY[0]), &(Bxd[0]), &(Byd[0]), 1.0);
			if((colinear = CheckColinear(&(BX[0]), &(BY[0]), Bpts)) != 0) {
				continue;
			}
			if(doshrink) {
				scale = CheckOutside();
				scale += EPS;
			} else {
				scale = 1.0;
			}
			MakeLongPts(Apts, &(AX[0]), &(AY[0]), &(Axd[0]), &(Ayd[0]), scale);
			if((colinear = CheckColinear(&(AX[0]), &(AY[0]), Apts)) != 0) {
				continue;
			}
			WriteProblem(fp);
			break;
		} else {
			Arad = 1000000.0 + frand()*(1000000.0);
			Brad = 100000.0 + frand()*(1000000.0);
			if(Brad > (Arad - 100.0)) Brad = (Arad - 100.0);
			r = Arad - Brad;
			r = (0.25*r + 0.5*r*frand());
			ang = 2.0*M_PI*frand();
			c = cos(ang);
			s = sin(ang);
			X = r*c;
			Y = r*s;
			r = Brad - sqrt(X*X + Y*Y);
			r = (0.25*r + 0.5*r*frand());
			ang = 2.0*M_PI*frand();
			c = cos(ang);
			s = sin(ang);
			X1 = X + r*c;
			Y1 = Y + r*s;
			r = Arad - Brad;
			r = (0.25*r + 0.5*r*frand());
			ang = 2.0*M_PI*frand();
			c = cos(ang);
			s = sin(ang);
			X2 = r*c;
			Y2 = r*s;
			MakeRandangles(&(Aang[0]), Apts);
			MakeRandangles(&(Bang[0]), Bpts);
			MakeDblPts(Apts, &(Axd[0]), &(Ayd[0]), &(Aang[0]), Arad, 0.0, 0.0, X2, Y2);
			MakeDblPts(Bpts, &(Bxd[0]), &(Byd[0]), &(Bang[0]), Brad, X, Y, X1, Y1);
			MakeLongPts(Bpts, &(BX[0]), &(BY[0]), &(Bxd[0]), &(Byd[0]), 1.0);
			if((colinear = CheckColinear(&(BX[0]), &(BY[0]), Bpts)) != 0) {
				continue;
			}
			scale = CheckOutside();
			if((scale < 1.0) && (doshrink == 0)) {
				scale = 1.0;
			} else {
				scale += EPS;
			}
			MakeLongPts(Apts, &(AX[0]), &(AY[0]), &(Axd[0]), &(Ayd[0]), scale);
			if((colinear = CheckColinear(&(AX[0]), &(AY[0]), Apts)) != 0) {
				continue;
			}
			WriteProblem(fp);
			break;
		}
	}
	fclose(fp);
	if(i >= 200) {
		return -20;
	}

	return i;
}