// viewpolysView.cpp : implementation of the CViewpolysView class
//

#include "stdafx.h"
#include "viewpolys.h"

#include "viewpolysDoc.h"
#include "viewpolysView.h"
#include "InFileDlg.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CViewpolysView

IMPLEMENT_DYNCREATE(CViewpolysView, CView)

BEGIN_MESSAGE_MAP(CViewpolysView, CView)
	//{{AFX_MSG_MAP(CViewpolysView)
	ON_COMMAND(ID_EDIT_SETINFILE, OnEditSetInFile)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CViewpolysView construction/destruction

CViewpolysView::CViewpolysView()
{
	m_bFileLoaded = FALSE;

}

CViewpolysView::~CViewpolysView()
{
}

BOOL CViewpolysView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CViewpolysView drawing

void CViewpolysView::OnDraw(CDC* pDC)
{
	CViewpolysDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CRect rect;
	int i, xcen, ycen;
	double dxcen, dycen, scale;
	CPen *pOldPen, bpen(PS_SOLID, 1, RGB(0,0,0)), rpen(PS_SOLID, 1, RGB(255,0,0));
	GetClientRect(&rect);
	pDC->FillSolidRect(&rect, RGB(255,255,255));
	if(m_bFileLoaded) {
		scale = ((double)(rect.Width() - 10))/(Axmax - Axmin);
		dxcen = ((double)(rect.Height() - 10))/(Aymax - Aymin);
		if(dxcen < scale) scale = dxcen;
		dxcen = 0.5*(Axmax + Axmin);
		dycen = 0.5*(Aymax + Aymin);
		xcen = rect.Width()/2;
		ycen = rect.Height()/2;
		for(i = 0; i < aVerts; i++) {
			Apts[i].x = xcen + (int)(scale*(A[i][0] - dxcen));
			Apts[i].y = ycen - (int)(scale*(A[i][1] - dycen));
		}
		pOldPen = pDC->SelectObject(&bpen);
		pDC->Polygon(Apts, aVerts);
		for(i = 0; i < bVerts; i++) {
			Bpts[i].x = xcen + (int)(scale*(B[i][0] - dxcen));
			Bpts[i].y = ycen - (int)(scale*(B[i][1] - dycen));
		}
		pOldPen = pDC->SelectObject(&rpen);
		pDC->Polygon(Bpts, bVerts);
		pDC->SelectObject(pOldPen);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CViewpolysView printing

BOOL CViewpolysView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CViewpolysView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CViewpolysView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CViewpolysView diagnostics

#ifdef _DEBUG
void CViewpolysView::AssertValid() const
{
	CView::AssertValid();
}

void CViewpolysView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CViewpolysDoc* CViewpolysView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CViewpolysDoc)));
	return (CViewpolysDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CViewpolysView message handlers

void CViewpolysView::OnEditSetInFile() 
{
	CInFileDlg dlg;	
	dlg.m_szInFile = m_szInFile;
	dlg.m_bReload = FALSE;
	if(dlg.DoModal() == IDOK) {
		if(dlg.m_bReload || (m_szInFile.CompareNoCase(dlg.m_szInFile) != 0)) {
			if(LoadFile(dlg.m_szInFile) == 0) {
				m_szInFile = dlg.m_szInFile;
			} else {
				LoadFile(m_szInFile);
			}
		}
		Invalidate();
		UpdateWindow();
	}
}

int CViewpolysView::LoadFile(LPCTSTR filePath)
{
	TCHAR inbuf[256];
	int i;
	FILE *fp;
	CString msg;

	if(::_tfopen_s(&(fp), filePath, _T("rt")) != 0){
		::AfxMessageBox(_T("Cannot open input file"));
		return -20;
	}
	m_bFileLoaded = FALSE;
	Axmax = Aymax = -1.0e10;
	Axmin = Aymin = 1.0e10;

	if(::_fgetts(&(inbuf[0]), _countof(inbuf), fp) == NULL){
		::AfxMessageBox(_T("read failed on A size"));
		::fclose(fp);
		return -1;
	}
	if(::_stscanf_s(&(inbuf[0]), _T("%d"), &(aVerts)) != 1 || (aVerts < 3) || (aVerts > 200)) {
		::AfxMessageBox(_T("scan failed on A size or bad value"));
		::fclose(fp);
		return -2;
	}
	for(i = 0; i < aVerts ; i++) {
		if (::_fgetts(&(inbuf[0]), _countof(inbuf), fp) == NULL) {
			msg.Format(_T("read failed on A vertex %d"), i+1);
			::AfxMessageBox(msg);
			fclose(fp);
			return -3;
		}
		if (::_stscanf_s(&(inbuf[0]), _T("%lf %lf"), &(A[i][0]), &(A[i][1])) != 2) {
			msg.Format(_T("scan failed on A vertex %d"), i+1);
			::AfxMessageBox(msg);
			::fclose(fp);
			return -4;
		}
		if(A[i][0] > Axmax) Axmax = A[i][0];
		if(A[i][0] < Axmin) Axmin = A[i][0];
		if(A[i][1] > Aymax) Aymax = A[i][1];
		if(A[i][1] < Aymin) Aymin = A[i][1];
	}
	// make wrapping A easier
	A[aVerts][0] = A[0][0];
	A[aVerts][1] = A[0][1];
	// get B
	if (::_fgetts(&(inbuf[0]), _countof(inbuf), fp) == NULL) {
		::AfxMessageBox(_T("read failed on B size"));
		::fclose(fp);
		return -5;
	}
	if (::_stscanf_s(&(inbuf[0]), _T("%d"), &(bVerts)) != 1 || (bVerts < 3) || (bVerts > 200)) {
		::AfxMessageBox(_T("scan failed on B size or bad value"));
		::fclose(fp);
		return -6;
	}
	for(i = 0; i < bVerts ; i++) {
		if (::_fgetts(&(inbuf[0]), _countof(inbuf), fp) == NULL) {
			msg.Format(_T("read failed on A vertex %d"), i + 1);
			::AfxMessageBox(msg);
			fclose(fp);
			return -7;
		}
		if (::_stscanf_s(&(inbuf[0]), _T("%lf %lf"), &(B[i][0]), &(B[i][1])) != 2) {
			msg.Format(_T("scan failed on B vertex %d"), i + 1);
			::AfxMessageBox(msg);
			::fclose(fp);
			return -8;
		}
	}
	// make wrapping B easier
	B[bVerts][0] = B[0][0];
	B[bVerts][1] = B[0][1];
	::fclose(fp);
	m_bFileLoaded = TRUE;
	return 0;
}
